/*
 
  LANChat Linux version 1.1.0 fully compatible with windows LC40
  Based on LC 4.0 specification by Piotr Kowalski <goro@vc.pl> 
  
  (c) Mateusz Skoczewski <melun@xox.pl>
  Antoby <antoby@mail.com>
  jeremian <jeremian@poczta.fm>
 
  ----------------------------------------------------------------------
  The Linux LANChat and related utilities are free software; you can
  redistribute it and/or modify it under the terms of the GNU Library
  general Public License as published by the Free Software Foundation;
  either version 2 of the License, or (at your option) any later version.
 
*/



#ifndef LANCHAT_LINUX_H
#define LANCHAT_LINUX_H

#include <ncurses.h>
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <pthread.h>

/*USER CUSTOMIZATION AREA*/

/* define below your operating system (max. 20 chars)*/
#define OS "Linux"

/*END OF USER CUSTOMIZATION AREA*/

/* current linux lanchat version*/
#define VER "1.1.0"

/* internal boradcast IP address*/
#define LC_BROADCAST 255

#define BUF		4096
#define LAG		100
#define AUTOALIVE	5000

/* lanchat messages*/
#define LC_NEWTOPIC 50
#define LC_TOPIC 53
#define LC_TOPIC_TIME 54
#define LC_CANTPOP 55
#define LC_POPOK 57
#define LC_POPUP 62
#define LC_MOTHER 78
#define LC_NORMAL 100
#define LC_AWAY 101
#define LC_ALIVE 102
#define LC_AWAY_ALIVE 115
#define LC_NEWNICK 116
#define LC_ME 104
#define LC_JOIN 107
#define LC_LEAVE 108
#define LC_REQ_AWAY 110
#define LC_REQ_ALIVE 111
#define LC_START_AWAY 113
#define LC_END_AWAY 114
#define LC_REQ_STATUS_EX 90
#define LC_STATUS_EX 91
#define LC_SLC_OK 92
#define LC_ADD_TO_ROOM 96
#define LC_REMOVE_FROM_ROOM 98
#define LC_START_WRITING 71
#define LC_STOP_WRITING 72

/* port*/
#define INPORT 16128

/* crypt method*/
#define CRYPT_DEFAULT 14

struct USER {
    char nick[20];
    unsigned char away;
    unsigned char room;    
    unsigned char lag;    
    unsigned char writing;

};

struct LINES {
	char* text;
	struct LINES* next;
	struct LINES* prev;
};

typedef struct LINES lines;

struct USER users[255];

struct LCHEAD {
        char cHead[4];
        unsigned char type;
        unsigned int sender;
        unsigned int dest;
        char nick[20];
        unsigned char nR;
        unsigned char nG;
        unsigned char nB;
        unsigned char xVal;
        unsigned char cCrypt;   
        unsigned char Female;
	unsigned long Reserved1;
        unsigned long Reserved2;
        unsigned long Reserved3;
        char data[4096];
} hsnd, hrcv, hhlp, hsnd2;


/************** function prototypes & macros ***************/

#define cmp(x,y) strncasecmp(x,y,strlen(y))

#define snd() send1=sendto(ssock,&hsnd,sizeof(hsnd)-4089+strlen(hsnd.data),\
0,(struct sockaddr *)&toaddr,sizeof(toaddr));pthread_mutex_lock(&sendmutex);\
bytes_snd+=send1;++total_snd;pthread_mutex_unlock(&sendmutex);

#define snd3() send3=sendto(ssock,&hsnd2,sizeof(hsnd2)-4089+strlen(hsnd2.data),\
0,(struct sockaddr *)&toaddr,sizeof(toaddr));pthread_mutex_lock(&sendmutex);\
bytes_snd+=send3;++total_snd;pthread_mutex_unlock(&sendmutex);


#define snd2() send2=sendto(ssock,&hhlp,sizeof(hhlp)-4089+strlen(hhlp.data),\
0,(struct sockaddr *)&toaddr,sizeof(toaddr));pthread_mutex_lock(&sendmutex);\
bytes_snd+=send2;++total_snd;pthread_mutex_unlock(&sendmutex);


extern void usage(void);
extern void cleanup(void);
extern void updatewin(WINDOW *);
extern void updateall(void);
extern char *atime(int);
extern char *adate(void);
extern int auser(char *);
extern void use_exit(int);
extern int checkstr(char *, char *);
extern void mcrypt(char *, int, int, unsigned char* );
extern void decrypt(int, char *, int, int);
extern void make_userlist(void);
extern void decr(void);
extern void sline(void);
extern void flog(char*);
extern void logtime(void);
extern void showtime(int);
extern void showtime2(void);
extern void show_normal(void);
extern void show_status(void);
extern void show_me(void);
extern void show_away(void);
extern void show_pop(void);
extern void show_msg(void);
extern void show_join(void);
extern void change_topic(void);
extern void show_leave(void);
extern void show_stats(void);
extern void show_help(void);
extern void show_credits(void);
extern void show_resurrected(void);
extern void show_unknown(void);
extern void show_went_away(void);
extern void show_back_away(void);
extern int prepare_message(void);
extern void sleeping(int);
extern void writetodef(char*, short);
extern void updatedefwin(void);
extern void cleandefwin(void);
extern void savingoptions(void);
extern void findword(void);
extern void show_features(void);
extern int iswithoutcomma(char*);


/******* global variables in lc_main.c *********/

extern WINDOW *inputwin, *defwin, *headwin, *split1win, *split2win, *split3win, *userwin, *split4win, *timewin;
extern char host[ 30 ], logname[ 20 ], myip[ 16 ], present[ 11 ], away[ 250 ],
     topic[ 75 ], topictime[ 9 ], readbuffer[ 4096 ], directory[ 200 ];
extern unsigned int  send_size, pop_mode;
extern unsigned long bytes_rcv, total_msg, nor_rcv, msg_rcv, pop_rcv;
extern unsigned long bytes_snd, total_snd, nor_snd, msg_snd, pop_snd;
extern FILE* f;
extern int pom;
extern long int czas;
extern pthread_t first, second;
extern pthread_mutex_t defmutex, writemutex, sendmutex;
extern int nlines, ncolumns;
extern int** defscreen;
extern int actwrli, actreli;
extern int backpos, fullheap;
extern char* line;
extern char writeus;
extern int chist, mhist;
extern unsigned char col_r, col_g, col_b;
extern char commands[18][9];

#endif /* LANCHAT_LINUX_H */
